/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified 
 *  version of the Library in object code or as an executable provided 
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete 
 *   corresponding machine-readable source code for the modified version 
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer 
 *   for a complete machine-readable copy of the corresponding source 
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#ifndef MARS_WORKLOAD_TYPES_H
#define MARS_WORKLOAD_TYPES_H

#if defined(__cplusplus)
extern "C" {
#endif

#include <stdint.h>
#include "mars/mars_task_types.h"

#define MARS_WORKLOAD_TYPE_UNKNOWN		0x0	/* workload undefined */
#define MARS_WORKLOAD_TYPE_TASK			0x1	/* workload type task */

#define MARS_WORKLOAD_ID_NONE			0xffff	/* workload id none */

#define MARS_WORKLOAD_STATE_NONE		0x00	/* workload undefined */
#define MARS_WORKLOAD_STATE_READY		0x01	/* ready to schedule */
#define MARS_WORKLOAD_STATE_WAITING		0x02	/* waiting for sync */
#define MARS_WORKLOAD_STATE_RUNNING		0x04	/* currently running */
#define MARS_WORKLOAD_STATE_FINISHED		0x08	/* not allow schedule */

#define MARS_WORKLOAD_PRIORITY_MIN		0x00	/* minimum priority */
#define MARS_WORKLOAD_PRIORITY_MAX		0xff	/* maximum priority */

#define MARS_WORKLOAD_MAX			1024	/* wl max */
#define MARS_WORKLOAD_PER_BLOCK			16	/* wl per block */
#define MARS_WORKLOAD_NUM_BLOCKS		64	/* wl max / per block */

#define MARS_WORKLOAD_CONTEXT_ALIGN		128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_HEADER_ALIGN	128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_BLOCK_ALIGN		128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_ALIGN		128	/* align to 128 bytes */

/* mars workload context */
struct mars_workload_context {
	uint8_t type;
	union {
		struct mars_task_context task;
	} type_context;
} __attribute__((aligned(MARS_WORKLOAD_CONTEXT_ALIGN)));

/* 128 byte workload queue header structure */
struct mars_workload_queue_header {
	uint64_t queue_ea;
	uint64_t context_ea;
	uint32_t count;
	uint8_t flag;
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_HEADER_ALIGN)));

/* 128 byte workload queue block structure */
struct mars_workload_queue_block {
	uint8_t state[MARS_WORKLOAD_PER_BLOCK];
	uint8_t priority[MARS_WORKLOAD_PER_BLOCK];
	uint8_t counter[MARS_WORKLOAD_PER_BLOCK];
	uint8_t signal[MARS_WORKLOAD_PER_BLOCK];
	uint16_t wait[MARS_WORKLOAD_PER_BLOCK];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_BLOCK_ALIGN)));

/* mars workload queue structure */
struct mars_workload_queue {
	struct mars_workload_queue_header header;
	struct mars_workload_queue_block block[MARS_WORKLOAD_NUM_BLOCKS];
	struct mars_workload_context context[MARS_WORKLOAD_MAX];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_ALIGN)));

#if defined(__cplusplus)
}
#endif

#endif
