/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified 
 *  version of the Library in object code or as an executable provided 
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete 
 *   corresponding machine-readable source code for the modified version 
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer 
 *   for a complete machine-readable copy of the corresponding source 
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#ifndef MARS_MUTEX_H
#define MARS_MUTEX_H

/**
 * \file
 * \ingroup group_mars_mutex
 * \brief [host] MARS Mutex API
 */

#if defined(__cplusplus)
extern "C" {
#endif

#include "mars/mars_mutex_types.h"

/**
 * \ingroup group_mars_mutex
 * \brief [host] Initializes a mutex.
 *
 * This function initializes a mutex instance that can be locked or unlocked
 * from both host and MPU to restrict concurrent accesses.
 *
 * \param[in] mutex		- pointer to mutex instance to initialize
 * \return
 *	MARS_SUCCESS		- successfully initialized mutex
 * \n	MARS_ERROR_NULL		- null pointer is specified
 * \n	MARS_ERROR_ALIGN	- instance not aligned properly
 */
int mars_mutex_initialize(struct mars_mutex *mutex);

/**
 * \ingroup group_mars_mutex
 * \brief [host] Locks a mutex.
 *
 * This function locks a mutex and blocks other requests to lock it.
 *
 * \param[in] mutex		- pointer to mutex instance to lock
 * \return
 *	MARS_SUCCESS		- successfully locked mutex
 * \n	MARS_ERROR_NULL		- null pointer is specified
 * \n	MARS_ERROR_ALIGN	- instance not aligned properly
 */
int mars_mutex_lock(struct mars_mutex *mutex);

/**
 * \ingroup group_mars_mutex
 * \brief [host] Unlocks a mutex.
 *
 * This function unlocks a previously locked mutex to allow other lock requests.
 *
 * \param[in] mutex		- pointer to mutex instance to unlock
 * \return
 *	MARS_SUCCESS		- successfully unlocked mutex
 * \n	MARS_ERROR_NULL		- null pointer is specified
 * \n	MARS_ERROR_ALIGN	- instance not aligned properly
 * \n	MARS_ERROR_STATE	- instance not in locked state
 */
int mars_mutex_unlock(struct mars_mutex *mutex);

#if defined(__cplusplus)
}
#endif

#endif
