/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified 
 *  version of the Library in object code or as an executable provided 
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete 
 *   corresponding machine-readable source code for the modified version 
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer 
 *   for a complete machine-readable copy of the corresponding source 
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#ifndef MARS_ATOMIC_H
#define MARS_ATOMIC_H

#if defined(__cplusplus)
extern "C" {
#endif

#include <spu_mfcio.h>
#include "mars/mars_debug.h"

#define MARS_ATOMIC_ALIGNMENT_MASK	0x7f

static inline void atomic_event_setup(void)
{
	/* clear phantom events and set event mask */
	if (spu_stat_event_status())
		spu_write_event_ack(spu_read_event_status());
	spu_write_event_mask(MFC_LLR_LOST_EVENT);
}

static inline void atomic_event_restore(void)
{
	/* clean up any remnant events, as needed */
	spu_write_event_mask(0);

	if (spu_stat_event_status()) {
		spu_read_event_status();
		spu_write_event_ack(MFC_LLR_LOST_EVENT);
	}
}

static inline void atomic_event_wait(void)
{
	/* wait until reservation is lost */
	spu_read_event_status();
	spu_write_event_ack(MFC_LLR_LOST_EVENT);
}

static inline void atomic_get(void *ls, uint64_t ea)
{
	MARS_ASSERT(((uintptr_t)ls & MARS_ATOMIC_ALIGNMENT_MASK) == 0);
	MARS_ASSERT((ea & MARS_ATOMIC_ALIGNMENT_MASK) == 0);

	mfc_getllar(ls, ea, 0, 0);
	mfc_read_atomic_status();
}

static inline int atomic_put(void *ls, uint64_t ea)
{
	MARS_ASSERT(((uintptr_t)ls & MARS_ATOMIC_ALIGNMENT_MASK) == 0);
	MARS_ASSERT((ea & MARS_ATOMIC_ALIGNMENT_MASK) == 0);

	spu_dsync();
	mfc_putllc(ls, ea, 0, 0);
	return mfc_read_atomic_status() & MFC_PUTLLC_STATUS;
}

#if defined(__cplusplus)
}
#endif

#endif
