/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified 
 *  version of the Library in object code or as an executable provided 
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete 
 *   corresponding machine-readable source code for the modified version 
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer 
 *   for a complete machine-readable copy of the corresponding source 
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#include <elf.h>

#include "mars/mars_elf.h"
#include "mars/mars_error.h"
#include "mars/mars_debug.h"

int mars_elf_parse(const void *elf_image, uint64_t *exec, uint32_t *exec_size,
			uint32_t *bss_size, uint32_t *vaddr, uint32_t *entry)
{
	MARS_CHECK_RET(elf_image, MARS_ERROR_NULL);
	MARS_CHECK_RET(exec, MARS_ERROR_NULL);
	MARS_CHECK_RET(exec_size, MARS_ERROR_NULL);
	MARS_CHECK_RET(bss_size, MARS_ERROR_NULL);
	MARS_CHECK_RET(vaddr, MARS_ERROR_NULL);
	MARS_CHECK_RET(entry, MARS_ERROR_NULL);

	Elf32_Ehdr *ehdr = (Elf32_Ehdr *)elf_image;
	Elf32_Phdr *phdr = (Elf32_Phdr *)((void *)ehdr + ehdr->e_phoff);

	//MARS_CHECK_RET(ehdr->e_phnum == 1, MARS_ERROR_FORMAT);
	MARS_CHECK_RET(phdr->p_type == PT_LOAD, MARS_ERROR_FORMAT);
	MARS_CHECK_RET(phdr->p_vaddr == MARS_ELF_VADDR,
			MARS_ERROR_FORMAT);

	/* set the context exec size and entry */
	*exec = (uint64_t)(uintptr_t)((void *)ehdr + phdr->p_offset);
	*exec_size = phdr->p_filesz;
	*bss_size = phdr->p_memsz - phdr->p_filesz;
	*vaddr = phdr->p_vaddr;
	*entry = ehdr->e_entry;

	return MARS_SUCCESS;
}
